"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Repo = void 0;
const base64 = require("@zodash/base64");
class Repo {
    constructor(ctx) {
        this.ctx = ctx;
    }
    getOwner(options) {
        var _a, _b;
        const owner = (_a = options === null || options === void 0 ? void 0 : options.owner) !== null && _a !== void 0 ? _a : (_b = this.ctx.core.options) === null || _b === void 0 ? void 0 : _b.owner;
        if (!owner) {
            throw new Error(`owner is required`);
        }
        return owner;
    }
    getRepo(options) {
        var _a, _b, _c;
        const repo = (_a = options === null || options === void 0 ? void 0 : options.repo) !== null && _a !== void 0 ? _a : (_c = (_b = this.ctx.core) === null || _b === void 0 ? void 0 : _b.options) === null || _c === void 0 ? void 0 : _c.repo;
        if (!repo) {
            throw new Error(`repo is required`);
        }
        return repo;
    }
    getBranch(options) {
        var _a, _b, _c;
        return (_a = options === null || options === void 0 ? void 0 : options.branch) !== null && _a !== void 0 ? _a : (_c = (_b = this.ctx.core) === null || _b === void 0 ? void 0 : _b.options) === null || _c === void 0 ? void 0 : _c.branch;
    }
    getPath(path) {
        return !/^\//.test(path) ? path : path.slice(1);
    }
    getMessage(action, path) {
        return `chore(${this.ctx.core.name}): ${action} file => ${path}`;
    }
    getContent(content) {
        if (!content) {
            throw new Error(`content is required`);
        }
        if (this.ctx.core.isBase64(content)) {
            return content;
        }
        return base64.encode(content);
    }
    async getSha(path, options) {
        if (options === null || options === void 0 ? void 0 : options.sha)
            return options.sha;
        const file = await this.getFile(path, options);
        return file.sha;
    }
    async hasFile(path, options) {
        try {
            await this.getFile(path, options);
            return true;
        }
        catch (error) {
            return false;
        }
    }
    async getFile(path, options) {
        const owner = this.getOwner(options);
        const repo = this.getRepo(options);
        const branch = this.getBranch(options);
        const response = await this.ctx.core.request('GET /repos/{owner}/{repo}/contents/{path}', {
            owner,
            repo,
            branch,
            path: this.getPath(path),
        });
        if (!(response === null || response === void 0 ? void 0 : response.data)) {
            throw new Error(`Cannot found file at ${path} in ${owner}/${repo}`);
        }
        return response.data;
    }
    async createFile(path, content, options) {
        var _a;
        const owner = this.getOwner(options);
        const repo = this.getRepo(options);
        const branch = this.getBranch(options);
        const message = (_a = options === null || options === void 0 ? void 0 : options.message) !== null && _a !== void 0 ? _a : this.getMessage('create', path);
        const response = await this.ctx.core.request('PUT /repos/{owner}/{repo}/contents/{path}', {
            owner,
            repo,
            branch,
            path: this.getPath(path),
            content: this.getContent(content),
            message,
            sha: options === null || options === void 0 ? void 0 : options.sha,
        });
        return response;
    }
    async updateFile(path, content, options) {
        var _a;
        const message = (_a = options === null || options === void 0 ? void 0 : options.message) !== null && _a !== void 0 ? _a : this.getMessage('update', path);
        const sha = await this.getSha(path, options);
        return this.createFile(path, content, { ...options, message, sha });
    }
    async deleteFile(path, options) {
        var _a;
        const owner = this.getOwner(options);
        const repo = this.getRepo(options);
        const branch = this.getBranch(options);
        const message = (_a = options === null || options === void 0 ? void 0 : options.message) !== null && _a !== void 0 ? _a : this.getMessage('delete', path);
        const sha = await this.getSha(path, options);
        return this.ctx.core.request('DELETE /repos/{owner}/{repo}/contents/{path}', {
            owner,
            repo,
            branch,
            path: this.getPath(path),
            sha,
            message,
        });
    }
    async listFiles(path, options) {
        const owner = this.getOwner(options);
        const repo = this.getRepo(options);
        const branch = this.getBranch(options);
        const _path = !path ? null : this.getPath(path);
        // try {
        const response = await this.ctx.core.request('GET /repos/{owner}/{repo}/git/trees/{tree_sha}?recursive=1', {
            owner,
            repo,
            tree_sha: branch !== null && branch !== void 0 ? branch : 'master',
        });
        const list = response.data.tree;
        if (_path === null) {
            return list;
        }
        if (_path === '') {
            return list.filter((e) => !e.path.includes('/'));
        }
        return list
            .filter((e) => e.path.startsWith(_path))
            .map((e) => {
            return {
                ...e,
                path: e.path.replace(_path + '/', ''),
            };
        })
            .filter((e) => !e.path.includes('/'));
        // } catch (error) {
        //   console.log(error.request);
        // }
    }
}
exports.Repo = Repo;
